Package: EFAtools
Title: Fast and Flexible Implementations of Exploratory Factor Analysis
        Tools
Version: 1.1.0
Authors@R: 
    c(person(given = "Markus",
             family = "Steiner",
             role = c("aut", "cre"),
             email = "markus.d.steiner@gmail.com",
             comment = c(ORCID = "0000-0002-8126-0757")),
      person(given = "Silvia",
             family = "Steiner",
             role = "aut",
             email = "silvia.steiner.grieder@gmail.com",
             comment = c(ORCID = "0000-0002-0118-7722")),
      person(given = "William",
             family = "Revelle",
             role = "ctb"),
      person(given = "Max",
             family = "Auerswald",
             role = "ctb"),
      person(given = "Morten",
             family = "Moshagen",
             role = "ctb"),
      person(given = "John",
             family = "Ruscio",
             role = "ctb"),
      person(given = "Brendan",
             family = "Roche",
             role = "ctb"),
      person(given = "Urbano",
             family = "Lorenzo-Seva",
             role = "ctb"),
      person(given = "David",
             family = "Navarro-Gonzalez",
             role = "ctb"),
      person(given = "Johan",
             family = "Braeken",
             role = "ctb"),
      person(given = "Andreas",
             family = "Soteriades",
             role = "ctb"))
Description: Provides a complete workflow for exploratory factor analysis
    (EFA). It covers data screening and factorability checks, a suite of
    factor retention criteria for choosing the number of factors, and factor
    extraction by principal axis factoring, maximum likelihood, unweighted
    least squares, or diagonally weighted least squares from Pearson,
    Spearman, Kendall, polychoric, tetrachoric, or two-stage
    full-information maximum likelihood correlations. A built-in rotation
    engine offers a range of orthogonal and oblique rotations, and standard
    errors for loadings and related quantities can be obtained by analytic,
    robust, or bootstrap methods. Further tools support model averaging
    across analytic choices, multigroup EFA with factor congruence, EFA on
    multiply imputed data, Schmid-Leiman transformation, reliability
    coefficients (including McDonald's omegas), factor score estimation,
    data simulation, and power analysis. Computationally intensive
    procedures are implemented in 'C++' for speed.
Depends: R (>= 4.1.0)
License: GPL-3
Encoding: UTF-8
LazyData: true
URL: https://github.com/mdsteiner/EFAtools,
        https://mdsteiner.github.io/EFAtools/
BugReports: https://github.com/mdsteiner/EFAtools/issues
Imports: psych, stats, ggplot2 (>= 3.4.0), cli, Rcpp, future.apply,
        checkmate, progressr, rlang, clue, lifecycle
LinkingTo: Rcpp, RcppArmadillo, roptim
Suggests: testthat (>= 3.0.0), future, GPArotation (>= 2022.4-1),
        lavaan, lavaan.mi, MASS, mice, nFactors, knitr, rmarkdown,
        vdiffr, polycor, mnormt, semTools, withr
VignetteBuilder: knitr
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-08-20 19:53:40 UTC; ma1159938
Author: Markus Steiner [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8126-0757>),
  Silvia Steiner [aut] (ORCID: <https://orcid.org/0000-0002-0118-7722>),
  William Revelle [ctb],
  Max Auerswald [ctb],
  Morten Moshagen [ctb],
  John Ruscio [ctb],
  Brendan Roche [ctb],
  Urbano Lorenzo-Seva [ctb],
  David Navarro-Gonzalez [ctb],
  Johan Braeken [ctb],
  Andreas Soteriades [ctb]
Maintainer: Markus Steiner <markus.d.steiner@gmail.com>
Repository: CRAN
Date/Publication: 2026-08-21 12:20:02 UTC
