# Generated by roxygen2: do not edit by hand

S3method(format,OMEGA)
S3method(format,efa)
S3method(format,efa_average)
S3method(format,efa_bartlett)
S3method(format,efa_compare)
S3method(format,efa_estimate_control)
S3method(format,efa_group)
S3method(format,efa_kmo)
S3method(format,efa_loadings)
S3method(format,efa_mi)
S3method(format,efa_power)
S3method(format,efa_reliability)
S3method(format,efa_retain)
S3method(format,efa_retention)
S3method(format,efa_rotate_control)
S3method(format,efa_schmid_leiman)
S3method(format,efa_scores)
S3method(format,efa_screen)
S3method(format,efa_simulated)
S3method(format,efa_sl_loadings)
S3method(format,summary.efa)
S3method(format,summary.efa_scores)
S3method(plot,efa_average)
S3method(plot,efa_compare)
S3method(plot,efa_group)
S3method(plot,efa_power)
S3method(plot,efa_retain)
S3method(plot,efa_retention)
S3method(print,OMEGA)
S3method(print,efa)
S3method(print,efa_average)
S3method(print,efa_bartlett)
S3method(print,efa_compare)
S3method(print,efa_estimate_control)
S3method(print,efa_group)
S3method(print,efa_kmo)
S3method(print,efa_loadings)
S3method(print,efa_mi)
S3method(print,efa_power)
S3method(print,efa_reliability)
S3method(print,efa_retain)
S3method(print,efa_retention)
S3method(print,efa_rotate_control)
S3method(print,efa_schmid_leiman)
S3method(print,efa_scores)
S3method(print,efa_screen)
S3method(print,efa_simulated)
S3method(print,efa_sl_loadings)
S3method(print,summary.efa)
S3method(print,summary.efa_scores)
S3method(residuals,efa)
S3method(summary,efa)
S3method(summary,efa_mi)
S3method(summary,efa_scores)
export(BARTLETT)
export(CD)
export(COMPARE)
export(EFA)
export(EFA_AVERAGE)
export(EFA_POOLED)
export(EKC)
export(FACTOR_SCORES)
export(HULL)
export(KGC)
export(KMO)
export(MAP)
export(NEST)
export(N_FACTORS)
export(OMEGA)
export(PARALLEL)
export(PROCRUSTES)
export(SCREE)
export(SL)
export(SMT)
export(efa_average)
export(efa_bartlett)
export(efa_cd)
export(efa_compare)
export(efa_ekc)
export(efa_fit)
export(efa_group)
export(efa_hull)
export(efa_kgc)
export(efa_kmo)
export(efa_map)
export(efa_mi)
export(efa_nest)
export(efa_parallel)
export(efa_power)
export(efa_procrustes)
export(efa_reliability)
export(efa_retain)
export(efa_schmid_leiman)
export(efa_scores)
export(efa_scree)
export(efa_screen)
export(efa_simulate)
export(efa_smt)
export(estimate_control)
export(rotate_control)
importFrom(Rcpp,sourceCpp)
importFrom(rlang,.data)
importFrom(utils,modifyList)
useDynLib(EFAtools, .registration = TRUE)
