#' UPPS_raw
#'
#' A dataframe containing responses to the UPPS personality scale (Whiteside &
#' Lynam, 2005) of 645 participants of Study 2 of Steiner and Frey (2020). Each
#' column are the ratings to one of 45 items to assess urgency, premeditation,
#' perseverance, and sensation seeking. The original data can be accessed via
#' <https://osf.io/kxp8t/>.
#'
#' @format A data.frame with 645 rows (participants) and 45 columns, named by a
#' subscale prefix and item number, covering the four UPPS subscales:
#' \describe{
#'   \item{perseverance_1 to perseverance_10}{(numeric) - Perseverance-subscale items.}
#'   \item{premeditation_1 to premeditation_11}{(numeric) - Premeditation-subscale items.}
#'   \item{ss_1 to ss_12}{(numeric) - Sensation-seeking-subscale items.}
#'   \item{urgency_1 to urgency_12}{(numeric) - Urgency-subscale items.}
#'  }
#' @source Whiteside, S. P., Lynam, D. R., Miller, J. D., & Reynolds, S. K. (2005).
#'  Validation of the UPPS impulsive behaviour scale: A four-factor model of
#'  impulsivity. European Journal of Personality, 19 (7), 559–574.
#' @source Steiner, M., & Frey, R. (2020). Representative design in psychological assessment: A case study using the Balloon Analogue Risk Task (BART). PsyArXiv Preprint. doi:10.31234/osf.io/dg4ks
"UPPS_raw"
