#' DOSPERT
#'
#' A list containing the bivariate correlations (cormat) of the 40 items of
#' the Domain Specific Risk Taking Scale (DOSPERT; Weber, Blais, & Betz, 2002)
#' and the sample size (N) based on the publicly available dataset at
#' (<https://osf.io/rce7g>) of the Basel-Berlin Risk Study (Frey et al., 2017).
#' The items measure risk-taking propensity on six different domains: social,
#' recreational, gambling, health/ safety, investment, and ethical.
#'
#' The underlying data deposit is licensed under CC BY 4.0
#' (<https://creativecommons.org/licenses/by/4.0/legalcode>). These correlations are a
#' derivative of it and are attributed to Frey et al. (2017), as the licence requires.
#'
#' @format A list of 2 with elements "cormat" (40 x 40 matrix of bivariate
#' correlations) and "N" (scalar).
#' \describe{
#'   \item{cormat}{(matrix) - Bivariate correlations of the 40 DOSPERT items,
#'   which span the six risk domains (social, recreational, gambling,
#'   health/safety, investment, and ethical).}
#'   \item{N}{(numeric) - The sample size the correlations are based on.}
#'  }
#' @source Weber, E. U., Blais, A.-R., & Betz, N. E. (2002). A domain specific risk-attitude scale: Measuring risk perceptions and risk behaviors. Journal of Behavioral Decision Making, 15(4), 263–290. doi: 10.1002/bdm.414
#' @source Frey, R., Pedroni, A., Mata, R., Rieskamp, J., & Hertwig, R. (2017). Risk preference shares the psychometric structure of major psychological traits. Science Advances, 3, e1701381.
#'
#' <https://osf.io/rce7g>
"DOSPERT"
