# print.efa_simulated summarises the object

    Code
      print(sim)
    Output
      -- Simulated data --------------------------------------------------------------
      
      100 cases by 18 variables (continuous).
      Marginals: normal.
      
      Model error: CB
      RMSEA: target <num>, achieved <num>
      CFI: target --, achieved <num>

---

    Code
      print(rp)
    Output
      -- Simulated data --------------------------------------------------------------
      
      Population correlation matrix only (18 variables); no data drawn.
      Marginals: normal.
      
      Model error: none (exact population).

