# the SPSS varimax non-convergence warning names the tolerance alone

    Code
      cat(conditionMessage(w))
    Output
      The rotation did not converge within 1000 iterations.
      i In `rotate_control()`, increase `precision` for a weaker tolerance.

