# print output is stable

    Code
      print(res)
    Output
      
      Total variance from the correlation matrix.
      
      -- Reliability coefficients ----------------------------------------------------
      
           tot  hier   sub  alpha    H
      g <num> <num> <num> <num> <num>
      F1 <num> <num> <num> <num> <num>
      F2 <num> <num> <num> <num> <num>
      F3 <num> <num> <num> <num> <num>
      
      -- Common-variance indices -----------------------------------------------------
      
          ECV   PUC
      g <num> <num>

---

    Code
      print(res_mg)
    Output
      
      Total variance from the correlation matrix.
      
      Group GroupA:
      
      -- Reliability coefficients ----------------------------------------------------
      
           tot  hier   sub  alpha    H
      g <num> <num> <num> <num> <num>
      F1 <num> <num> <num> <num> <num>
      F2 <num> <num> <num> <num> <num>
      F3 <num> <num> <num> <num> <num>
      
      -- Common-variance indices -----------------------------------------------------
      
          ECV   PUC
      g <num> <num>
      
      Group GroupB:
      
      -- Reliability coefficients ----------------------------------------------------
      
           tot  hier   sub  alpha    H
      g <num> <num> <num> <num> <num>
      F1 <num> <num> <num> <num> <num>
      F2 <num> <num> <num> <num> <num>
      F3 <num> <num> <num> <num> <num>
      
      -- Common-variance indices -----------------------------------------------------
      
          ECV   PUC
      g <num> <num>

---

    Code
      print(res_sf)
    Output
      
      Total variance from the model-implied composite variance.
      
      -- Reliability coefficients ----------------------------------------------------
      
          tot    H
      g <num> <num>

