% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efa_retain.R
\name{format.efa_retain}
\alias{format.efa_retain}
\title{Format method for efa_retain objects}
\usage{
\method{format}{efa_retain}(x, ...)
}
\arguments{
\item{x}{an object of class efa_retain, returned by \code{\link[=efa_retain]{efa_retain()}}.}

\item{...}{not used.}
}
\value{
A character vector with the report lines (styled to the active
console theme; plain when colours are disabled).
}
\description{
Format method for efa_retain objects
}
\examples{
\donttest{
nf <- efa_retain(test_models$baseline$cormat, criteria = c("EKC", "SMT"),
                 N = 500)
writeLines(format(nf))
}
}
