% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.BARTLETT.R
\name{print.BARTLETT}
\alias{print.BARTLETT}
\title{Print BARTLETT object}
\usage{
\method{print}{BARTLETT}(x, ...)
}
\arguments{
\item{x}{list of class BARTLETT (output from the \link{BARTLETT} function)}

\item{...}{additional arguments passed to print}
}
\description{
Print BARTLETT object
}
\examples{
BARTLETT(test_models$baseline$cormat, N = 500)

}
