% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.PARALLEL.R
\name{plot.PARALLEL}
\alias{plot.PARALLEL}
\title{Plot PARALLEL object}
\usage{
\method{plot}{PARALLEL}(x, ...)
}
\arguments{
\item{x}{list of class PARALLEL. An output from the \link{PARALLEL} function.}

\item{...}{not used.}
}
\description{
Plot method showing a summarized output of the \link{PARALLEL} function
}
\examples{
\donttest{
# example with correlation matrix and "ML" estimation
x <- PARALLEL(test_models$case_11b$cormat, N = 500, method = "ML")
plot(x)
}
}
