% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.KGC.R
\name{plot.KGC}
\alias{plot.KGC}
\title{Plot KGC object}
\usage{
\method{plot}{KGC}(x, ...)
}
\arguments{
\item{x}{a list of class KGC. An output from the \link{KGC} function.}

\item{...}{not used.}
}
\description{
Plot method showing a summarized output of the \link{KGC} function
}
\examples{
KGC_base <- KGC(test_models$baseline$cormat)
plot(KGC_base)

}
