% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WJIV_ages_3_5_doc.R
\docType{data}
\name{WJIV_ages_3_5}
\alias{WJIV_ages_3_5}
\title{Woodcock Johnson IV: ages 3 to 5}
\format{
A list of 2 with elements "cormat" (29 x 29 matrix of bivariate correlations)
and "N" (scalar). The correlation matrix contains the following variables:
\describe{
\item{ORLVOC}{(numeric) - Oral Vocabulary.}
\item{VRBATN}{(numeric) - Verbal Attention.}
\item{LETPAT}{(numeric) - Phonological Processing.}
\item{STYREC}{(numeric) - Story Recall.}
\item{VISUAL}{(numeric) - Visualization.}
\item{GENINF}{(numeric) - General Information.}
\item{CONFRM}{(numeric) - Concept Formation.}
\item{NUMREV}{(numeric) - Numbers Reversed.}
\item{NUMPAT}{(numeric) - Number-Pattern Matching.}
\item{NWDREP}{(numeric) - Nonword Repetition.}
\item{VAL}{(numeric) - Visual-Auditory Learning.}
\item{PICREC}{(numeric) - Picture Recognition.}
\item{MEMWRD}{(numeric) - Memory for Words.}
\item{PICVOC}{(numeric) - Picture Vocabulary.}
\item{ORLCMP}{(numeric) - Oral Comprehension.}
\item{SEGMNT}{(numeric) - Segmentation.}
\item{RPCNAM}{(numeric) - Rapid Picture Naming.}
\item{SENREP}{(numeric) - Sentence Repetition.}
\item{UNDDIR}{(numeric) - Understanding Directions.}
\item{SNDBLN}{(numeric) - Sound Blending.}
\item{RETFLU}{(numeric) - Retrieval Fluency.}
\item{SNDAWR}{(numeric) - Sound Awareness.}
\item{LWIDNT}{(numeric) - Letter-Word Identification.}
\item{APPROB}{(numeric) - Applied Problems.}
\item{SPELL}{(numeric) - Spelling.}
\item{PSGCMP}{(numeric) - Passage Comprehension.}
\item{SCI}{(numeric) - Science.}
\item{SOC}{(numeric) - Social Studies.}
\item{HUM}{(numeric) - Humanities.}
}
}
\source{
McGrew, K. S., LaForte, E. M., & Schrank, F. A. (2014). Technical
Manual. Woodcock-Johnson IV. Rolling Meadows, IL: Riverside.

Schrank, F. A., McGrew, K. S., & Mather, N. (2014). Woodcock-Johnson IV.
Rolling Meadows, IL: Riverside.
}
\usage{
WJIV_ages_3_5
}
\description{
A list containing the bivariate correlations (N = 435) of the 29
cognitive and achievement subtests from the WJ IV for 3- to 5-year-olds from
the standardization sample obtained from
the WJ IV technical Manual (McGrew, LaForte, & Schrank, 2014).
Tables are reproduced with permission from the publisher.
}
\keyword{datasets}
