% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.SL.R
\name{print.SL}
\alias{print.SL}
\title{Print SL object}
\usage{
\method{print}{SL}(x, ...)
}
\arguments{
\item{x}{list. An object of class SL to be printed}

\item{...}{Further arguments for print.}
}
\description{
Print Method showing a summarized output of the \link{SL} function.
}
\examples{
EFA_mod <- EFA(test_models$baseline$cormat, N = 500, n_factors = 3,
               type = "EFAtools", method = "PAF", rotation = "promax")
SL(EFA_mod, type = "EFAtools", method = "PAF")

}
