% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.HULL.R
\name{print.HULL}
\alias{print.HULL}
\title{Print function for HULL objects}
\usage{
\method{print}{HULL}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class HULL. Output from the \code{\link{HULL}} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for HULL objects
}
\examples{
\donttest{
HULL(test_models$baseline$cormat, N = 500, method = "ML")
}
}
