% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.change_class}
\alias{.change_class}
\title{Convert an \code{"efa_loadings"} table to matrix or a matrix to \code{"efa_loadings"}}
\usage{
.change_class(x, cl = "matrix")
}
\arguments{
\item{x}{A table of class \code{"matrix"} or \code{"efa_loadings"}.}

\item{cl}{A character vector with the class to change the table to. Should be
\code{c("efa_loadings", "LOADINGS")} or \code{"matrix"}.}
}
\value{
A table with the loadings, of class either \code{"efa_loadings"} or
\code{"matrix"}.
}
\description{
The loadings tables returned by \code{\link[=efa_fit]{efa_fit()}} are of class
\code{c("efa_loadings", "LOADINGS")}, which prevents applying functions on them.
This function changes their class to \code{"matrix"}, and changes it back to
\code{"efa_loadings"} when done.
}
\author{
Andreas Soteriades
}
\keyword{internal}
