% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.KGC.R
\name{print.KGC}
\alias{print.KGC}
\title{Print function for KGC objects}
\usage{
\method{print}{KGC}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class KGC. Output from \link{KGC} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for KGC objects
}
\examples{
KGC_base <- KGC(test_models$baseline$cormat)
KGC_base

}
