% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efa_retention.R
\name{format.efa_retention}
\alias{format.efa_retention}
\title{Format method for efa_retention objects}
\usage{
\method{format}{efa_retention}(x, ...)
}
\arguments{
\item{x}{an object of class efa_retention, returned by a factor-retention
criterion (e.g. \code{\link[=efa_ekc]{efa_ekc()}} or \code{\link[=efa_hull]{efa_hull()}}).}

\item{...}{not used.}
}
\value{
A character vector with the report lines (styled to the active
console theme; plain when colours are disabled).
}
\description{
Format method for efa_retention objects
}
\examples{
writeLines(format(efa_ekc(test_models$baseline$cormat, N = 500)))
}
