% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.PARALLEL.R
\name{print.PARALLEL}
\alias{print.PARALLEL}
\title{Print function for PARALLEL objects}
\usage{
\method{print}{PARALLEL}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class PARALLEL. Output from \link{PARALLEL} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for PARALLEL objects
}
\examples{
\donttest{
# example without real data
PARALLEL(N = 500, n_vars = 10)

# example with correlation matrix and "ML" estimation
PARALLEL(test_models$case_11b$cormat, N = 500, method = "ML")
}
}
