% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.extract_list_object}
\alias{.extract_list_object}
\title{Extract a list object by its name}
\usage{
.extract_list_object(alist, object)
}
\arguments{
\item{alist}{A list of sub-lists, typically a list of \eqn{m} objects of class
\code{"efa"}, where \eqn{m} is the number of imputations passed to
\code{\link[=efa_mi]{efa_mi()}}.}

\item{object}{String of length 1. The name of the object to extract e.g.
\code{"h2"} or \code{"vars_accounted"}.}
}
\value{
A list of length \eqn{m}, with each element containing the extracted
\code{object} for the \eqn{k}th element (\eqn{k = 1,..., m}).
}
\description{
Consider a list of named sub-lists. This function extracts, for each sub-list,
the sub-list element that is specified by the user. This function is useful
for extracting results from \code{\link[=efa_fit]{efa_fit()}} for each imputation in
\code{\link[=efa_mi]{efa_mi()}}.
}
\author{
Andreas Soteriades
}
\keyword{internal}
