% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.EKC.R
\name{plot.EKC}
\alias{plot.EKC}
\title{Plot EKC object}
\usage{
\method{plot}{EKC}(x, ...)
}
\arguments{
\item{x}{a list of class EKC. An output from the \link{EKC} function.}

\item{...}{not used.}
}
\description{
Plot method showing a summarized output of the \link{EKC} function
}
\examples{
EKC_base <- EKC(test_models$baseline$cormat, N = 500)
plot(EKC_base)

}
