% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.SLLOADINGS.R
\name{print.SLLOADINGS}
\alias{print.SLLOADINGS}
\title{Print SLLOADINGS object}
\usage{
\method{print}{SLLOADINGS}(x, cutoff = 0.2, digits = 3, ...)
}
\arguments{
\item{x}{class SLLOADINGS matrix.}

\item{cutoff}{numeric. The number above which to print loadings in bold
(default is .2).}

\item{digits}{numeric. Passed to \code{\link[base:Round]{round}}. Number of digits
to round the loadings to (default is 3).}

\item{...}{additional arguments passed to print}
}
\description{
Print SLLOADINGS object
}
\examples{
EFA_mod <- EFA(test_models$baseline$cormat, N = 500, n_factors = 3,
               type = "EFAtools", method = "PAF", rotation = "promax")
SL(EFA_mod, type = "EFAtools", method = "PAF")

}
