% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.KMO.R
\name{print.KMO}
\alias{print.KMO}
\title{Print KMO object}
\usage{
\method{print}{KMO}(x, ...)
}
\arguments{
\item{x}{list of class KMO (output from the \link{KMO} function)}

\item{...}{additional arguments passed to print}
}
\description{
Print KMO object
}
\examples{
KMO_base <- KMO(test_models$baseline$cormat)
KMO_base

}
