% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.average_matrices}
\alias{.average_matrices}
\title{Average a list of matrices elementwise}
\usage{
.average_matrices(x)
}
\arguments{
\item{x}{List of conformable matrices.}
}
\value{
A matrix with the same dimensions as the inputs.
}
\description{
Average a list of matrices elementwise
}
