% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.EKC.R
\name{print.EKC}
\alias{print.EKC}
\title{Print function for EKC objects}
\usage{
\method{print}{EKC}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class EKC. Output from \code{\link{EKC}} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for EKC objects
}
\examples{
EKC_base <- EKC(test_models$baseline$cormat, N = 500)
EKC_base

}
