% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.MAP.R
\name{print.MAP}
\alias{print.MAP}
\title{Print function for MAP objects}
\usage{
\method{print}{MAP}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class MAP. Output from \code{\link{MAP}} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for MAP objects
}
\examples{
MAP_base <- MAP(test_models$baseline$cormat)
MAP_base

}
