% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.EFA_AVERAGE.R
\name{print.EFA_AVERAGE}
\alias{print.EFA_AVERAGE}
\title{Print EFA_AVERAGE object}
\usage{
\method{print}{EFA_AVERAGE}(x, stat = c("average", "range"), plot = TRUE, ...)
}
\arguments{
\item{x}{list. An object of class EFA_AVERAGE to be printed}

\item{stat}{character. A vector with the statistics to print. Possible inputs
are "average", "sd", "range", "min", and "max". Default is "average" and
"range".}

\item{plot}{logical. Whether a plot of the average and min- max loadings should
be created. Default is TRUE. If more than 10 factors are extracted, no plot is
created.}

\item{...}{Further arguments for print.}
}
\description{
Print Method showing a summarized output of the \link{EFA_AVERAGE} function
}
\examples{
\dontrun{
EFA_aver <- EFA_AVERAGE(test_models$baseline$cormat, n_factors = 3, N = 500)
EFA_aver
}
}
