% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.SMT.R
\name{print.SMT}
\alias{print.SMT}
\title{Print SMT object}
\usage{
\method{print}{SMT}(x, ...)
}
\arguments{
\item{x}{list of class SMT (output from the \link{SMT} function)}

\item{...}{additional arguments passed to print}
}
\description{
Print SMT object
}
\examples{
SMT_base <- SMT(test_models$baseline$cormat, N = 500)
SMT_base

}
