% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.numformat}
\alias{.numformat}
\title{Format numbers for print method}
\usage{
.numformat(x, digits = 2, print_zero = FALSE)
}
\arguments{
\item{x}{numeric. Number to be formatted.}

\item{digits}{numeric. Number of digits after the comma to keep.}

\item{print_zero}{logical. Whether, if a number is between [-1, 1], the
zero should be omitted or printed (default is FALSE, i.e. omit zeros).}
}
\value{
A formated number
}
\description{
Helper function used in the print method for class LOADINGS and SLLOADINGS.
Strips the 0 in front of the decimal point of a number if number < 1, only
keeps the first \code{digits} number of digits, and adds an empty space in
front of the number if the number is positive. This way all returned strings
(except for those > 1, which are exceptions in LOADINGS) have the same number
of characters.
}
