% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.NEST.R
\name{print.NEST}
\alias{print.NEST}
\title{Print function for NEST objects}
\usage{
\method{print}{NEST}(x, ...)
}
\arguments{
\item{x}{a list of class NEST Output from \code{\link{NEST}} function.}

\item{...}{Further arguments for print.}
}
\description{
Print function for NEST objects
}
\examples{
NEST(test_models$baseline$cormat, N = 500)

}
