% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.N_FACTORS.R
\name{print.N_FACTORS}
\alias{print.N_FACTORS}
\title{Print function for N_FACTORS objects}
\usage{
\method{print}{N_FACTORS}(x, ...)
}
\arguments{
\item{x}{a list of class N_FACTORS. Output from \link{N_FACTORS} function.}

\item{...}{Further arguments for print.}
}
\description{
Print function for N_FACTORS objects
}
\examples{
\donttest{
# All criteria except "CD", with correlation matrix and fit method "ML"
# (where needed)
N_FACTORS(test_models$baseline$cormat, criteria = c("EKC", "HULL", "KGC",
          "PARALLEL", "SCREE", "SMT"), N = 500, method = "ML")
}
}
