% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.stat_over_list}
\alias{.stat_over_list}
\title{Calculate statistics for a list of matrices}
\usage{
.stat_over_list(alist, stat)
}
\arguments{
\item{alist}{A list of sub-lists, typically a list of \eqn{m} matrices from
\code{"EFA"}, where \eqn{m} is the number of imputations passed to
\code{\link{EFA_POOLED}}.}

\item{stat}{A function, e.g. \code{mean} or \code{sd}.}
}
\value{
A matrix with the aggregated results
}
\description{
Calculate statistics for a list of matrices
}
\author{
Andreas Soteriades

Given a list of matrices, this function calculates user-supplied statistics
(e.g. mean, median) over the matrices. This function is useful for averaging
results from \code{\link{EFA}} (e.g. loadings) over all permutation runs in
\code{\link{EFA_POOLED}}.
}
