% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{.nest_sym}
\alias{.nest_sym}
\title{Get reference values for nest.}
\usage{
.nest_sym(nf, N, M, nreps = 1000L)
}
\arguments{
\item{nf}{numeric. Current number of factors.}

\item{N}{numeric. Number of cases / observations in dataset.}

\item{M}{numeric matrix. A transposed matrix of cbind(loadings, diag(sqrt(uniquenesses))).}

\item{nreps}{numeric. Number of datasets to create.}
}
\description{
Function called from within NEST so usually no call to this is needed by the user.
Provides a C++ implementation of the NEST simulation procedure
}
