% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GRiPS_raw_doc.R
\docType{data}
\name{GRiPS_raw}
\alias{GRiPS_raw}
\title{GRiPS_raw}
\format{
A data.frame with 810 rows (participants) and 8 columns, one per GRiPS
item. Each item is a self-report indicator of general risk propensity, labelled
by a keyword from the item:
\describe{
\item{fun}{(numeric) - Risk-taking makes life more fun.}
\item{friends}{(numeric) - Friends would describe the respondent as a risk taker.}
\item{enjoy}{(numeric) - Enjoyment of taking risks.}
\item{hurt}{(numeric) - Willingness to take a risk even if it might hurt.}
\item{part}{(numeric) - Risk-taking as an important part of life.}
\item{commonly}{(numeric) - Commonly takes risks.}
\item{chances}{(numeric) - Belief in taking chances.}
\item{attracted}{(numeric) - Attracted, rather than scared, by risk.}
}
}
\source{
Zhang, D. C., Highhouse, S., & Nye, C. D. (2019). Development and validation of the general risk propensity scale (GRiPS). Journal of Behavioral Decision Making, 32, 152–167. doi: 10.1002/bdm.2102

Steiner, M., & Frey, R. (2020). Representative design in psychological assessment: A case study using the Balloon Analogue Risk Task (BART). PsyArXiv Preprint. doi:10.31234/osf.io/dg4ks
}
\usage{
GRiPS_raw
}
\description{
A data.frame containing responses to the General Risk Propensity Scale (GRiPS, Zhang,
Highhouse & Nye, 2018) of 810 participants of Study 1 of Steiner and Frey (2020).
The original data can be accessed via \url{https://osf.io/kxp8t/}.
}
\keyword{datasets}
