% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.HULL.R
\name{plot.HULL}
\alias{plot.HULL}
\title{Plot HULL object}
\usage{
\method{plot}{HULL}(x, ...)
}
\arguments{
\item{x}{list of class HULL. An output from the \code{\link{HULL}} function.}

\item{...}{not used.}
}
\description{
Plot method showing a summarized output of the \code{\link{HULL}} function
}
\examples{
\donttest{
x <- HULL(test_models$baseline$cormat, N = 500, method = "ML")
plot(x)
}
}
