% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.SCREE.R
\name{print.SCREE}
\alias{print.SCREE}
\title{Print function for SCREE objects}
\usage{
\method{print}{SCREE}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class SCREE Output from \link{SCREE} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for SCREE objects
}
\examples{
SCREE_base <- SCREE(test_models$baseline$cormat)
SCREE_base

}
