% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.CD.R
\name{plot.CD}
\alias{plot.CD}
\title{Plot CD object}
\usage{
\method{plot}{CD}(x, ...)
}
\arguments{
\item{x}{a list of class CD. An output from the \link{CD} function.}

\item{...}{not used.}
}
\description{
Plot method showing a summarized output of the \link{CD} function
}
