% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.CD.R
\name{print.CD}
\alias{print.CD}
\title{Print function for CD objects}
\usage{
\method{print}{CD}(x, plot = TRUE, ...)
}
\arguments{
\item{x}{a list of class CD. Output from \link{CD} function.}

\item{plot}{logical. Whether to plot the results.}

\item{...}{Further arguments for print.}
}
\description{
Print function for CD objects
}
\examples{
\donttest{
# determine n factors of the GRiPS
CD(GRiPS_raw)
}

}
