% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit-indices.R
\name{.compute_vars}
\alias{.compute_vars}
\title{Compute explained variances from loadings}
\usage{
.compute_vars(L_unrot, L_rot, Phi = NULL)
}
\arguments{
\item{L_unrot}{matrix. Unrotated factor loadings.}

\item{L_rot}{matrix. Rotated factor loadings.}

\item{Phi}{matrix. Factor intercorrelations. Provide only if oblique rotation
is used.}
}
\value{
A matrix with sum of squared loadings, proportion explained variance
from total variance per factor, same as previous but cumulative, Proportion
of explained variance from total explained variance, and same as previous but
cumulative. The three cumulative and common-variance rows are omitted when
\code{L_rot} has a single column, where they would only repeat the two above them,
so the result has two rows there and five otherwise.
}
\description{
From unrotated loadings compute the communalities and uniquenesses for total
variance. Compute explained variances per factor from rotated loadings (and
factor intercorrelations Phi if oblique rotation was used).
}
\keyword{internal}
