% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot.SCREE.R
\name{plot.SCREE}
\alias{plot.SCREE}
\title{Plot SCREE object}
\usage{
\method{plot}{SCREE}(x, ...)
}
\arguments{
\item{x}{a list of class SCREE An output from the \link{SCREE} function.}

\item{...}{not used.}
}
\description{
Plot method showing a summarized output of the \link{SCREE} function
}
\examples{
SCREE_base <- SCREE(test_models$baseline$cormat)
plot(SCREE_base)

}
