% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/alignment.R
\name{.tucker_congruence}
\alias{.tucker_congruence}
\title{Tucker congruence between factors}
\usage{
.tucker_congruence(L1, L2)
}
\arguments{
\item{L1}{Numeric matrix.}

\item{L2}{Numeric matrix with the same dimensions as \code{L1}.}
}
\value{
A square matrix whose \verb{(i, j)} entry is the Tucker congruence
between column \code{i} of \code{L1} and column \code{j} of \code{L2}.
}
\description{
Compute the Tucker congruence matrix between the columns of two loading
matrices.
}
\references{
Lorenzo-Seva, U., and ten Berge, J. M. F. (2006). Tucker's congruence
coefficient as a meaningful index of factor similarity. \emph{Methodology}, 2,
57-64.
}
\keyword{internal}
