% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.hyperplane_count}
\alias{.hyperplane_count}
\title{Count near-zero loadings}
\usage{
.hyperplane_count(L, cutoff = 0.15)
}
\arguments{
\item{L}{Numeric loading matrix.}

\item{cutoff}{Numeric scalar. Loadings with `abs(L) < cutoff` are counted as
being in the hyperplane.}
}
\value{
A list with the total hyperplane count and counts by factor and item.
}
\description{
Hyperplane count is the number of loadings with absolute value smaller than a
user-specified cutoff.
}
