% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/residuals.EFA.R
\name{residuals.EFA}
\alias{residuals.EFA}
\title{Residuals function for EFA objects}
\usage{
\method{residuals}{EFA}(object, print = TRUE, digits = 3, ...)
}
\arguments{
\item{object}{a list of class EFA. Output from \code{\link{EFA}} or
\code{\link{EFA_POOLED}}.}

\item{print}{logical. Whether to print residuals. If FALSE, they are just returned.}

\item{digits}{numeric. The number of digits to round printed residuals.}

\item{...}{Further arguments.}
}
\description{
Residuals function for EFA objects
}
