% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{.factor_corres}
\alias{.factor_corres}
\title{Compute number of non-matching indicator-to-factor correspondences}
\usage{
.factor_corres(x, y, thresh = 0.3)
}
\arguments{
\item{x}{numeric matrix. A matrix of pattern coefficients.}

\item{y}{numeric matrix. A second matrix of coefficients.}

\item{thresh}{numeric. The threshold to classify a pattern coefficient as substantial.}
}
\description{
Compute number of non-matching indicator-to-factor correspondences
}
\keyword{internal}
