% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/helper.R
\name{.consensus_loss}
\alias{.consensus_loss}
\title{Mean squared discrepancy to a consensus target}
\usage{
.consensus_loss(aligned_loadings, target)
}
\arguments{
\item{aligned_loadings}{List of aligned loading matrices.}

\item{target}{Consensus target matrix.}
}
\value{
Mean sum of squared deviations from the target across matrices.
}
\description{
Mean squared discrepancy to a consensus target
}
