% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EFAtools-package.R
\docType{package}
\name{EFAtools-package}
\alias{EFAtools}
\alias{EFAtools-package}
\title{EFAtools: Fast and Flexible Implementations of Exploratory Factor Analysis Tools}
\description{
Provides a complete workflow for exploratory factor analysis (EFA). It covers data screening and factorability checks, a suite of factor retention criteria for choosing the number of factors, and factor extraction by principal axis factoring, maximum likelihood, unweighted least squares, or diagonally weighted least squares from Pearson, Spearman, Kendall, polychoric, tetrachoric, or two-stage full-information maximum likelihood correlations. A built-in rotation engine offers a range of orthogonal and oblique rotations, and standard errors for loadings and related quantities can be obtained by analytic, robust, or bootstrap methods. Further tools support model averaging across analytic choices, multigroup EFA with factor congruence, EFA on multiply imputed data, Schmid-Leiman transformation, reliability coefficients (including McDonald's omegas), factor score estimation, data simulation, and power analysis. Computationally intensive procedures are implemented in 'C++' for speed.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/mdsteiner/EFAtools}
  \item \url{https://mdsteiner.github.io/EFAtools/}
  \item Report bugs at \url{https://github.com/mdsteiner/EFAtools/issues}
}

}
\author{
\strong{Maintainer}: Markus Steiner \email{markus.d.steiner@gmail.com} (\href{https://orcid.org/0000-0002-8126-0757}{ORCID})

Authors:
\itemize{
  \item Markus Steiner \email{markus.d.steiner@gmail.com} (\href{https://orcid.org/0000-0002-8126-0757}{ORCID})
  \item Silvia Steiner \email{silvia.steiner.grieder@gmail.com} (\href{https://orcid.org/0000-0002-0118-7722}{ORCID})
}

Other contributors:
\itemize{
  \item William Revelle [contributor]
  \item Max Auerswald [contributor]
  \item Morten Moshagen [contributor]
  \item John Ruscio [contributor]
  \item Brendan Roche [contributor]
  \item Urbano Lorenzo-Seva [contributor]
  \item David Navarro-Gonzalez [contributor]
  \item Johan Braeken [contributor]
  \item Andreas Soteriades [contributor]
}

}
\keyword{internal}
