% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/efa_retain.R
\name{print.efa_retain}
\alias{print.efa_retain}
\title{Print method for efa_retain objects}
\usage{
\method{print}{efa_retain}(x, ...)
}
\arguments{
\item{x}{an object of class efa_retain, returned by \code{\link[=efa_retain]{efa_retain()}}.}

\item{...}{not used.}
}
\value{
\code{print()} returns its argument \code{x} invisibly; it is
\code{cat(format(x), sep = "\\n")}.
}
\description{
Print method for efa_retain objects
}
\examples{
\donttest{
efa_retain(test_models$baseline$cormat, criteria = c("EKC", "SMT"), N = 500)
}
}
