% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/procrustes-consensus.R
\name{.orthogonal_procrustes}
\alias{.orthogonal_procrustes}
\title{Closed-form orthogonal Procrustes rotation}
\usage{
.orthogonal_procrustes(A, B)
}
\arguments{
\item{A}{Numeric matrix to be rotated.}

\item{B}{Numeric target matrix with the same dimensions as \code{A}.}
}
\value{
A list with the rotated loadings, orthogonal transformation matrix,
target criterion value, and basic diagnostics.
}
\description{
Rotate \code{A} to the orthogonal target \code{B} by minimizing
\verb{||A \%*\% T - B||_F^2} subject to \code{t(T) \%*\% T = I}.
}
\references{
Schoenemann, P. H. (1966). A generalized solution of the orthogonal
Procrustes problem. \emph{Psychometrika}, 31, 1-10.
}
\keyword{internal}
