% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EFAtools-superseded.R
\name{MAP}
\alias{MAP}
\title{Minimum average partial}
\usage{
MAP(
  x,
  use = c("pairwise.complete.obs", "all.obs", "complete.obs", "everything",
    "na.or.complete"),
  cor_method = c("pearson", "spearman", "kendall", "poly", "tetra")
)
}
\arguments{
\item{x}{A numeric \code{matrix} or \code{data.frame}. Can be either (a) a correlation matrix, or
(b) raw data (rows = observations, columns = variables) from which correlations are computed.}

\item{use}{Character string specifying the treatment of missing values when computing correlations.
Passed to \code{\link[stats:cor]{stats::cor()}}. Defaults to \code{"pairwise.complete.obs"}.}

\item{cor_method}{Character string specifying the correlation coefficient to be computed if raw
data are supplied. One of \code{"pearson"}, \code{"spearman"}, or \code{"kendall"} (passed to
\code{\link[stats:cor]{stats::cor()}}), or \code{"poly"} / \code{"tetra"} for polychoric / tetrachoric correlations
of ordinal / binary data (a two-step estimator). Defaults to \code{"pearson"}.}
}
\value{
An object of class \code{efa_retention}, identical to the value of
\code{\link[=efa_map]{efa_map()}}; see there for the components.
}
\description{
\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#superseded}{\figure{lifecycle-superseded.svg}{options: alt='[Superseded]'}}}{\strong{[Superseded]}}

\code{MAP()} has been superseded by \code{\link[=efa_map]{efa_map()}}, which is the recommended interface
going forward. It remains available and unchanged so existing code keeps working.
}
\seealso{
\code{\link[=efa_map]{efa_map()}}
}
